/* 
	InerziaLightning
	Calculates the distance from a thunderstorm by measuring the time
	between a lightning and the thunder.
	It takes account of the sound speed dependance on the air temperature
 */
var gTempValue = 0;             //valore temporaneo per il tempo di misura
var gInitTemp = 0;				//tempo della prima misura - per porre a 0 la prima
var gTimerAttivo = false;       //vero se timer attivo
var gTimerID = null;            //ID del timer
var gStepTimer = 150;           //intervallo di aggiornamento in ms
var gMilliSec = 0;				//ora di partenza del conteggio
var gSpeed = soundSpeed(20);	//velocità del suono
var gTemperatura = 20;			//temperatura attuale
var gTempUnit = 0;				//unità di misura temperatura attuale
var gUnit = 0;					//unità di misura attuale
var conversione = new Array(1,0.001,1.093613298,0.000621371192,0.000539956803,3.280839895); //fattori conversione unità lunghezza
var precisione = new Array(0, 2, 1, 2, 2, 0);			//cifre decimali a seconda delle unità
var testoUnit = new Array('metri','chilometri','yarde','miglia','migliaMarine','piedi'); //chiavi localizzazione unità lunghezza

var DIM_MEMORIA = 4;                    //numero posizioni di memoria
var distanzaVect = new Array();         //contiene le ultime n misure
var tempoVect = new Array();            //contiene gli istanti delle ultime n misure
	distanzaVect.length = DIM_MEMORIA;
	tempoVect.length = DIM_MEMORIA;
var gDati = -1;                         //numero di misure da considerare
var gMisure = 0;						//numero di misure consecutive
var MIN_RANGE_NULL = 300;				//range minimo oltre il quale il temporale si sta muovendo

var TEMPERATURE_DEFAULT = "temperatureDefault";     // costanti per preferenze nel plist
var TEMPERATURE_UNIT = "temperatureUnit";
var LENGTH_UNIT = "lengthDefaultUnit";

var DIST_LO = 900;                      // costanti distanza per i colori
var DIST_MID = 1800;

var versione = "2.0";                   // versione del widget



if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}

/* --- funzioni --- */

/*
 * Chiamata da click su tasto
 * se il timer non è attivo, fa partire il conteggio, altrimenti lo ferma
 * Cambia la label a seconda della funzione
 */
function avvia(event)
{
    if ( ! gTimerAttivo)
	{
		var partenza = new Date();
		gMilliSec = partenza.getTime();
		gTimerID = setInterval("conteggio()", gStepTimer);
		gTimerAttivo = true;
		delete partenza;
		document.getElementById('avviaButton').object.textElement.innerText = getLocalizedString('ferma');
		document.getElementById('showDistance').style.backgroundColor = "#ffffff";
		document.getElementById('clear').style.color = "#888888";
	}
	else
	{
		clearInterval(gTimerID);
		gTimerAttivo = false;
		document.getElementById('clear').style.color = "#000000";
		conteggio();
		document.getElementById('avviaButton').object.textElement.innerText = getLocalizedString('avvia');
		var distanza = Number(document.getElementById('showDistance').value)/conversione[gUnit];
        memoria(distanza,gTempValue/1000);		//mette in memoria distanza e tempo
        var direzione = bestFit();				//coefficiente angolare
		var rangeDist = deltaD();				//range
		if (distanza <= DIST_LO) {
			document.getElementById('showDistance').style.backgroundColor = "#ff3300";
		} else if (distanza>DIST_LO && distanza<DIST_MID) {
			document.getElementById('showDistance').style.backgroundColor = "#ffbb00";
		} else {
			document.getElementById('showDistance').style.backgroundColor = "#33ff00";
		}
		
		if (gDati > 0)
		{
			var temp = 'stable';
			var valore = 5;
			if ( rangeDist > MIN_RANGE_NULL )
			{
				temp =  (direzione > 0) ? 'away' : 'near';
				valore = (direzione > 0) ? 3 : 7;
			}
			document.getElementById("spia").object.setValue(valore);
			document.getElementById("direction").innerText = getLocalizedString(temp);
		}
		else
		{
			document.getElementById("spia").object.setValue(1);
			document.getElementById("direction").innerText = "";
		}
	}
}


/*
 *	Chiamata dal timer
 *	Ricava l'intervallo, lo trasforma in distanza
 *	e lo scrive nel display
 */
function conteggio()
{
	var adesso = new Date();
    gTempValue = adesso.getTime();
	var delta = gTempValue - gMilliSec;
	var distanza = delta * gSpeed * conversione[gUnit];
	document.getElementById('showDistance').value = distanza.toFixed( precisione[gUnit] );
	delete adesso;
}


/*
 *  Inserisce valore attuale della distanza
 *  e dei tempi
 */
function memoria(d,t)
{
    for (i=DIM_MEMORIA-1; i>0; i--)
    {
        distanzaVect[i] = distanzaVect[i-1];
        tempoVect[i] = tempoVect[i-1];
    }
    distanzaVect[0] = d;
	if (gDati == -1) gInitTemp = t;
    tempoVect[0] = t - gInitTemp;
    if (gDati < DIM_MEMORIA-1) gDati++;
	document.getElementById("misureFld").innerText = ++gMisure;
}

/*
 *	Effettua il best fit delle coppie tempo-distanza
 *	e calcola il coefficiente angolare della retta
 */
function bestFit()
{
    if (gDati == 0) { return 0 };
    if (gDati == 1) { return ((distanzaVect[1] - distanzaVect[0])/(tempoVect[1] - tempoVect[0])); }
    var n = gDati+1;
    var sx = 0;
    for (i=0; i<=gDati; i++) {sx += tempoVect[i];}
    var sy = 0;
    for (i=0; i<=gDati; i++) {sy += distanzaVect[i];}
    var sxy = 0;
    for (i=0; i<=gDati; i++) {sxy += distanzaVect[i] * tempoVect[i];}
    var sx2 = 0;
    for (i=0; i<=gDati; i++) {sx2 += tempoVect[i] * tempoVect[i];}
	return (n*sxy - sx*sy)/(n*sx2 - sx*sx);
}


/*
 *	Calcola il range max-min delle distanze
 */
function deltaD()
{
	var dMax = 0;
	var dMin = 99999;
	if ( gDati == 0 ) {return 0;}
	if ( gDati == 1 ) { return ( Math.max(distanzaVect[0],distanzaVect[1]) - Math.min(distanzaVect[0],distanzaVect[1]) ); }
	
	for (i=0; i<=gDati; i++)
	{
		if ( distanzaVect[i] > dMax ) { dMax = distanzaVect[i] }
		if ( distanzaVect[i] < dMin ) { dMin = distanzaVect[i] }
	}
	return ( dMax - dMin );
}


/*
 *	azzera le misure e riparte da zero
 *	sempre che il conteggio non sia attivo
 */
function azzera(event)
{
	if ( !gTimerAttivo )
	{
		gDati = -1;
		gMisure = 0;
		document.getElementById("spia").object.setValue(1);
		document.getElementById("direction").innerText = "";
		document.getElementById('showDistance').style.backgroundColor = "#ffffff";
		document.getElementById("spia").object.setValue(1);
		document.getElementById("misureFld").innerText = 0;
		document.getElementById("showDistance").value = 0;
	}
}


/*
 *	calcola la velocità del suono nell'aria in m/ms
 *	in funzione della temperatura in °C (trasformata se necessario)
 */
function soundSpeed(temp)
{
	var tempC = translate2C( temp, gTempUnit );
	return (331.4 + 0.62*tempC)/1000;
}


/*
 * Ferma il timer
 */
function stopTimer()
{
	if ( gTimerAttivo )
	{
		clearInterval(gTimerID);
		gTimerAttivo = false;
	}
}


/*
 * Riempie i settaggi del back con i valori attuali
 * dopo aver fermato il timer, se attivo
 */
function preparaBack()
{
	stopTimer();
	document.getElementById('tempFld').value = gTemperatura;
	document.getElementById('degreeCmb').selectedIndex = gTempUnit;
	document.getElementById('unitPrefCmb').selectedIndex = gUnit;

}

/*
 * Prende i settaggi del back, li fa diventare globali
 * e li salva nelle preferenze. Ricalcola la nuova velocità del suono.
 * Se nel display c'è qualcosa, lo trasforma nella nuova unità
 */
function preparaFront()
{
	gTemperatura = Number(document.getElementById('tempFld').value);
	gTempUnit = document.getElementById('degreeCmb').selectedIndex;
	gUnit = document.getElementById('unitPrefCmb').selectedIndex;
	
	widget.setPreferenceForKey(gTemperatura, TEMPERATURE_DEFAULT);
	widget.setPreferenceForKey(gTempUnit, TEMPERATURE_UNIT);
	widget.setPreferenceForKey(gUnit, LENGTH_UNIT);

	gSpeed = soundSpeed(gTemperatura);
	
	document.getElementById('unity').innerText = getLocalizedString( testoUnit[gUnit] ); //cambia nome unità lunghezza
	
	var valore = document.getElementById('showDistance').value;		//converte eventuale distanza in nuova lunghezza
	if ( valore != 0 )
	{
		document.getElementById('showDistance').value = (valore * conversione[gUnit]).toFixed( precisione[gUnit] );
	}
}


/*
 * Carica le preferenze dal plist
 * e calcola velocità suono; inserisce il numero di versione sul back
 */
function loadPreferences()
{
	if ( ! (widget.preferenceForKey(TEMPERATURE_DEFAULT) == undefined) )
	{
		gTemperatura = Number(widget.preferenceForKey(TEMPERATURE_DEFAULT));
	}
	gTempUnit = widget.preferenceForKey(TEMPERATURE_UNIT);
	gUnit = widget.preferenceForKey(LENGTH_UNIT);
	
	gSpeed = soundSpeed(gTemperatura);
	document.getElementById('unity').innerText = getLocalizedString( testoUnit[gUnit] );
	document.getElementById('versione').innerText = versione;
	document.getElementById('misureFld').innerText = 0;
}


/*
 * Chiamato dal cambio del combo delle temperature
 * Traduce il contenuto del campo temperatura
 * Le preferenze sono sempre salvate nel PreparaFront
 */
function tempComboChanged(event)
{
	var newTempUnit = document.getElementById('degreeCmb').selectedIndex;
	var presentTemp = Number(document.getElementById('tempFld').value);
	if ( newTempUnit != gTempUnit )
	{
		var tempC = translate2C( presentTemp, gTempUnit );
		var tempTemp = convertCTemp( tempC, newTempUnit );
		document.getElementById('tempFld').value = tempTemp.toFixed(0);
		gTempUnit = newTempUnit;
	}
}


/*
 * converte temperatura qualunque in °C; unitIndex è l'unità da cui vogliamo tradurre
 */
function translate2C( temp, unitIndex )
{
	var tempC;
	switch ( unitIndex )	{
		case 0:
			tempC = temp;
			break;
		case 1:
			tempC = 5*(temp - 32)/9;
			break;
		case 2:
			tempC = temp - 273.15;
			break;
	}
	return tempC;
}

/*
 * Converte °C nella scala attuale; unitIndex è l'unità verso cui tradurre
 */
function convertCTemp( tempC, unitIndex )
{
	var tempOther;
	switch (unitIndex)
	{
		case 0:
			tempOther = tempC;
			break;
		case 1:
			tempOther = 9*tempC/5 + 32;
			break;
		case 2:
			tempOther = tempC + 273.15;
			break;
	}
	return tempOther;
}


/*
 *	Link to the help onLine inerziasoft
 */
function helpOnLine(event)
{
    widget.openURL('http://www.inerziasoft.eu/');
}



//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    dashcode.setupParts();
	loadPreferences();
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
	stopTimer();
    // Stop any timers to prevent CPU usage
    // Remove any preferences as needed
    // widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    stopTimer();
	// Stop any timers to prevent CPU usage
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
    // Restart any timers that were stopped on hide
}

//
// Function: sync()
// Called when the widget has been synchronized with .Mac
//
function sync()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
	loadPreferences();
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    preparaBack();				/* codice aggiunto */
	
	var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
	
	/* codice aggiunto */
	var valore = document.getElementById('showDistance').value;			//trasforma in metri l'eventuale display
	if ( valore != 0 )
	{
		document.getElementById('showDistance').value = valore / conversione[gUnit];  
	}}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    preparaFront();				/* codice aggiunto */
	
	var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}
